"""This module containes LinkButtons"""
from .binder_button import BinderLinkButton
from .image_link_button import ImageLinkButton
from .nbviewer_button import NBViewerLinkButton
from .panel_button import PanelLinkButton



"""# PandasProfileReport

The PandasProfileReport pane enables a user to show a ProfileReport generated by the
[Pandas profile_report](https://github.com/pandas-profiling/pandas-profiling) package.
"""
import html

import panel as pn
import param
from pandas_profiling import ProfileReport

# pylint: disable=line-too-long
OBJECT_WHEN_LOADING_REPORT_REPORT = (
    "<p class='pandas-profile-report-loading'>Loading Report ...</p>"
)
GREEN = "#174c4f"
ORANGE = "#cc5c29"
LOGO_URL = "https://raw.githubusercontent.com/MarcSkovMadsen/awesome-panel/master/application/pages/pandas_profiling_app/pandas_profiler_logo.png"
# pylint: enable=line-too-long
STYLE = "width:100%;height:100%;"
OBJECT_WHEN_NO_REPORT = "<p class='pandas-profile-report-no-report'>No Report Available</p>"


class PandasProfileReport(pn.pane.HTML):
    """The PandasProfilingApp showcases how to integrate the Pandas Profiling Report with Panel"""

    profile_report = param.ClassSelector(class_=ProfileReport)
    object_when_loading_report = param.String(OBJECT_WHEN_LOADING_REPORT_REPORT)
    object_when_no_report = param.String(OBJECT_WHEN_NO_REPORT)

    # In order to not be selected by the `pn.panel` selection process
    # Cf. https://github.com/holoviz/panel/issues/1494#issuecomment-663219654
    priority = 0
    # The _rename dict is used to keep track of Panel parameters to sync to Bokeh properties.
    # As value is not a property on the Bokeh model we should set it to None
    _rename = dict(pn.pane.HTML._rename, profile_report=None)

    def __init__(self, **params):
        super().__init__(**params)

        self._update_object_from_parameters()

    # Don't name the function
    # `_update`, `_update_object`, `_update_model` or `_update_pane`
    # as this will override a function in the parent class.
    @param.depends("profile_report", "object_when_no_report", watch=True)
    def _update_object_from_parameters(self):
        if not self.profile_report:
            self.object = self.object_when_no_report
            return

        self.object = self.object_when_loading_report
        self.object = self._to_html(self.profile_report)

    @staticmethod
    def _to_html(profile_report: ProfileReport) -> str:
        html_report = profile_report.to_html()
        html_report = html.escape(html_report)
        return (
            f'<iframe srcdoc="{html_report}" style={STYLE} frameborder="0" '
            'allowfullscreen></iframe>'
        )

    def __str__(self):
        return "Pandas Profile Report"

    def __repr__(self, depth: int = 1):
        return self.__str__()



"""This module contains functionality used by awesome-panel in Notebooks"""
import panel as pn
import param

from awesome_panel_extensions.widgets.link_buttons import (
    BinderLinkButton,
    NBViewerLinkButton,
    PanelLinkButton,
)

pn.extension()

DEFAULT_HEADER_MESSAGE = """\
[Panel](https://panel.holoviz.org) is a framework for creating powerful, reactive analytics apps in
Python using the tools you know and love. 💪🐍❤️.
This notebook is brought to you by [awesome-panel.org](https://awesome-panel.org).
"""


class Header(pn.Column):
    """Extension Implementation"""

    def __init__(  # pylint: disable=too-many-arguments
        self,
        folder: str,
        notebook: str,
        repository: str = "marcskovmadsen/awesome-panel-extensions",
        branch: str = "master",
        message: str = DEFAULT_HEADER_MESSAGE,
        **params,
    ):
        params["sizing_mode"] = params.get("sizing_mode", "stretch_width")
        super().__init__(**params)
        self.binder_link_button = BinderLinkButton(
            repository=repository, branch=branch, folder=folder, notebook=notebook,
        )
        self.nbviewer_link_button = NBViewerLinkButton(
            repository=repository, branch=branch, folder=folder, notebook=notebook,
        )
        self.panel_link_button = PanelLinkButton(theme="dark")

        text = pn.pane.Markdown(message, sizing_mode="stretch_width")
        buttons = pn.Row(
            self.panel_link_button,
            self.binder_link_button,
            self.nbviewer_link_button,
            Style(container_width="90%"),
            sizing_mode="stretch_width",
        )
        self[:] = [buttons, text]


class Style(pn.pane.HTML):
    """Class for styling a reference notebook"""

    container_width = param.String(default="100%")

    # In order to not be selected by the `pn.panel` selection process
    # Cf. https://github.com/holoviz/panel/issues/1494#issuecomment-663219654
    priority = 0
    # The _rename dict is used to keep track of Panel parameters to sync to Bokeh properties.
    # As value is not a property on the Bokeh model we should set it to None
    _rename = {
        **pn.pane.HTML._rename,
        "container_width": None,
    }

    def __init__(self, **params):
        super().__init__(**params)
        self._update_object_from_parameters()

    # Don't name the function
    # `_update`, `_update_object`, `_update_model` or `_update_pane`
    # as this will override a function in the parent class.
    @param.depends("container_width", watch=True)
    def _update_object_from_parameters(self, *_):
        self.object = f"<style>.container {{ width:{self.container_width} !important; }}</style>"



"""The ImageLinkButton displays the image_url. When clicked the link_url is opened in a new tab"""
import param
from panel.pane.markup import HTML

_STYLE = {
    "cursor": "pointer",
    "border": "1px solid #ddd",
    "border-radius": "4px",
    "padding": "5px",
}


class ImageLinkButton(HTML):
    """The ImageLinkButton widget is a Link Button that
    - looks like the specified image_url
    - Open the link_url in a new tab when clicked"""
    image_url = param.String(default=None, doc="The url to the image")
    link_url = param.String(default=None, doc="The url to open when clicked")
    object = param.String(
        default=None,
        doc="""
        The object being wrapped, which will be converted to a
        Bokeh model.""",
        constant=True,
    )
    height = param.Integer(
        default=31,
        bounds=(0, None),
        doc="""
        The height of the component (in pixels).  This can be either
        fixed or preferred height, depending on height sizing policy.""",
    )
    width = param.Integer(
        default=300,
        bounds=(0, None),
        doc="""
        The width of the component (in pixels). This can be either
        fixed or preferred width, depending on width sizing policy.""",
    )

    # In order to not be selected by the `pn.panel` selection process
    # Cf. https://github.com/holoviz/panel/issues/1494#issuecomment-663219654
    priority = 0

    def __init__(self, **params):
        parent_parameters = [parameter.name for parameter in HTML.param.objects().values()]
        child_parameters = [parameter.name for parameter in self.param.objects().values()]
        parameters_to_rename = list(set(child_parameters) - set(parent_parameters))
        self._rename = self._rename.copy()
        self._rename.update({parameter: None for parameter in parameters_to_rename})

        params["style"] = params.get("style", {**_STYLE})

        super().__init__(**params)
        self._update_object_from_parameters()

    @param.depends("image_url", "link_url", watch=True)
    def _update_object_from_parameters(self, *_):
        with param.edit_constant(self):
            self.object = (
                f'<a href="{self.link_url}" target="_blank">'
                f'<img src="{self.image_url}" style="height:100%;max-width:100%;display:block;'
                'margin-left:auto;margin-right:auto"></a>'
            )


class DerivedImageLinkButton(ImageLinkButton):
    """Base Class for other Link Buttons like BinderLinkButton"""
    image_url = param.String(default=None, doc="The url to the image", constant=True)
    link_url = param.String(default=None, doc="The url to open when clicked", constant=True)

    @staticmethod
    def _html_encode(item: str) -> str:
        return item.replace("/", "%2F").replace("\\", "%2F")



"""Definition of Light and Dark Foreground and Background themes

According to Material Design Palette.

Source: https://github.com/angular/components/blob/master/src/material/core/theming/_palette.scss
"""
import param

from . import color
from .color_palette import GREY

MATERIAL_FONT_FAMILY = "roboto, sans-serif, Verdana"


class ThemeForeground(param.Parameterized):
    """Material Design Theme Foreground settings"""

    base = param.String(color.BLACK)
    divider = param.String(color.DARK_DIVIDERS)
    dividers = param.String(color.DARK_DIVIDERS)
    disabled = param.String(color.DARK_DISABLED_TEXT)
    disabled_button = param.String("rgba(0,0,0, 0.26)")
    disabled_text = param.String(color.DARK_DISABLED_TEXT)
    elevation = param.String(color.BLACK)
    hint_text = param.String(color.DARK_DISABLED_TEXT)
    primary_text = param.String(color.DARK_PRIMARY_TEXT)
    secondary_text = param.String(color.DARK_SECONDARY_TEXT)
    icon = param.String("rgba(0,0,0 , 0.54)")
    icons = param.String("rgba(0,0,0 , 0.54)")
    text = param.String("rgba(0,0,0 , 0.87)")
    slider_min = param.String("rgba(0,0,0 , 0.87)")
    slider_off = param.String("rgba(0,0,0 , 0.26)")
    slider_off_active = param.String("rgba(0,0,0 , 0.38)")

    def __eq__(self, other):
        return self.name == other.name


class ThemeBackground(param.Parameterized):
    """Material Design Theme Background settings"""

    status_bar = param.String(GREY.color_300)
    app_bar = param.String(GREY.color_100)
    background = param.String(GREY.color_50)
    hover = param.String("rgba(0,0,0, 0.04)")
    card = param.String(color.WHITE)
    dialog = param.String(color.WHITE)
    disabled_button = param.String("rgba(0,0,0, 0.12)")
    raised_button = param.String(color.WHITE)
    focused_button = param.String(color.DARK_FOCUSED)
    selected_button = param.String(GREY.color_300)
    selected_disabled_button = param.String(GREY.color_400)
    disabled_button_toggle = param.String(GREY.color_200)
    unselected_chip = param.String(GREY.color_300)
    disabled_list_option = param.String(GREY.color_200)
    tooltip = param.String(GREY.color_700)

    def __eq__(self, other):
        return self.name == other.name


LIGHT_THEME_FOREGROUND = ThemeForeground(name="Light Theme Foreground")
LIGHT_THEME_BACKGROUND = ThemeBackground(name="Light Theme Background")

DARK_THEME_FOREGROUND = ThemeForeground(
    name="Dark Theme Foreground",
    base=color.WHITE,
    divider=color.LIGHT_DIVIDERS,
    dividers=color.LIGHT_DIVIDERS,
    disabled=color.LIGHT_DISABLED_TEXT,
    disabled_button="rgba(255,255,255, 0.3)",
    disabled_text=color.LIGHT_DISABLED_TEXT,
    elevation=color.BLACK,
    hint_text=color.LIGHT_DISABLED_TEXT,
    primary_text=color.LIGHT_PRIMARY_TEXT,
    secondary_text=color.LIGHT_SECONDARY_TEXT,
    icon=color.WHITE,
    icons=color.WHITE,
    text=color.WHITE,
    slider_min=color.WHITE,
    slider_off="rgba(255,255,255, 0.3",
    slider_off_active="rgba(255, 255, 255, 0.3",
)

DARK_THEME_BACKGROUND = ThemeBackground(
    name="Dark Theme Background",
    status_bar=color.BLACK,
    app_bar=GREY.color_900,
    background="#121212",  # 303030
    hover="rgba(255,255,255, 0.04)",
    card=GREY.color_800,
    dialog=GREY.color_800,
    disabled_button="rgba(255,255,255, 0.12)",
    raised_button=GREY.color_800,
    focused_button=color.LIGHT_FOCUSED,
    selected_button=GREY.color_900,
    selected_disabled_button=GREY.color_800,
    disabled_button_toggle=color.BLACK,
    unselected_chip=GREY.color_700,
    disabled_list_option=color.BLACK,
    tooltip=GREY.color_700,
)
FOREGROUNDS = [LIGHT_THEME_FOREGROUND, DARK_THEME_FOREGROUND]
BACKGROUNDS = [LIGHT_THEME_BACKGROUND, DARK_THEME_BACKGROUND]


class Theme(param.Parameterized):
    """Material Design Theme Foreground and background settings"""
    foreground = param.ObjectSelector(LIGHT_THEME_FOREGROUND, objects=FOREGROUNDS)
    background = param.ObjectSelector(LIGHT_THEME_BACKGROUND, objects=BACKGROUNDS)

    def __eq__(self, other):
        return self.name == other.name


LIGHT_THEME = Theme(name="White Theme")
DARK_THEME = Theme(
    name="Dark Theme", foreground=DARK_THEME_FOREGROUND, background=DARK_THEME_BACKGROUND,
)

THEMES = [LIGHT_THEME, DARK_THEME]



"""Setup file for the Awesome Panel Extensions"""
from typing import List

import setuptools

# I only want to include a short README with a focus on the package
with open("README_PACKAGE.md", "r") as fh:
    long_description = fh.read()

install_requires = [
    "panel==0.9.7",
    "bokeh==2.1.1",
    "lxml",  # Required by the WebComponent
]

_recommended: List[str] = []

_tests = [
    "autoflake",
    "invoke",
    "isort",
    "jupyter-repo2docker",
    "mypy",
    "pylint",
    "pytest",
    "pytest-cov",
    "rope",
    "twine",
]

_examples = [
    "notebook",
    "pandas-profiling",
    "matplotlib",
    "pandas",
    "seaborn",
    "holoviews",
    "hvplot",
    "plotly",
    "altair",
]

_doc: List[str] = []

extras_require = {
    "examples": _recommended + _examples,
    "tests": _tests,
    "recommended": _recommended,
    "doc": _recommended + _doc,
}

extras_require["all"] = sorted(set(sum(extras_require.values(), [])))

setuptools.setup(
    name="awesome-panel-extensions",
    version="20200823.1",
    description="A package of awesome Panel extensions. Provided by awesome-panel.org",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Marc Skov Madsen",
    author_email="marc.skov.madsen@gmail.com",
    platforms=["Windows", "Mac OS X", "Linux"],
    license="GPLv3",
    url="https://github.com/marcskovmadsen/awesome-panel-extensions",
    # My Project contains more folders/ packages but they should not be included
    packages=setuptools.find_packages(
        include=["awesome_panel_extensions", "awesome_panel_extensions.*"]
    ),
    include_package_data=True,
    classifiers=[
        # I would like to indicate that this package is a package for the Panel framework
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Financial and Insurance Industry",
        "Intended Audience :: Healthcare Industry",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Legal Industry",
        "Intended Audience :: Other Audience",
        "Intended Audience :: Science/Research",
        "Natural Language :: English",
        "Topic :: Scientific/Engineering",
        "Topic :: Scientific/Engineering :: Visualization",
        "Topic :: Scientific/Engineering :: Information Analysis",
        "Topic :: Office/Business",
        "Topic :: Office/Business :: Financial",
        "Topic :: Software Development :: Libraries",
    ],
    python_requires=">=3.6",
    install_requires=install_requires,
    extras_require=extras_require,
    tests_require=extras_require["tests"],
)
